package be.iminds.ilabt.jfed.rspec;

import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonProperty;

import javax.annotation.Nonnull;
import javax.annotation.Nullable;
import java.util.*;

/**
 * config for AdvertisementBasedRspecGenerator
 */
public class AdvertisementBasedRspecGeneratorConfig {
    @Nonnull public final List<NodeInfo> nodes;

    @JsonCreator
    public AdvertisementBasedRspecGeneratorConfig(
            @Nonnull @JsonProperty("nodes") List<NodeInfo> nodes) {
        this.nodes = nodes;
    }

    @Nonnull
    public List<NodeInfo> getNodes() {
        return nodes;
    }

    public static class NodeInfo {
        public final int count;
        @Nonnull public final String clientIdBase;               //setter
        @Nullable public final String ansibleGroup;              //setter
        @Nullable public final String diskImage;                 //setter
        @Nullable public final List<String> componentIdSelector; //setter             (how to set componentID)
        @Nullable public final List<String> componentIdFilter;   //filter             (which componentIds are allowed)
        @Nullable public final String sliverType;                //filter AND setter
        @Nullable public final List<String> hardwareTypeFilter;  //filter
        @Nullable public final Boolean availableFilter;          //filter
        @Nullable public final Boolean exclusive;                //setter

        public NodeInfo(int count,
                        @Nullable String clientIdBase,
                        @Nullable String ansibleGroup,
                        @Nullable String diskImage,
                        @Nullable List<String> componentIdSelector,
                        @Nullable List<String> componentIdFilter,
                        @Nullable String sliverType,
                        @Nullable List<String> hardwareTypeFilter,
                        @Nullable Boolean availableFilter,
                        @Nullable Boolean exclusive) {
            this.count = count;
            this.clientIdBase = clientIdBase == null ? "node" : clientIdBase;
            this.ansibleGroup = ansibleGroup;
            this.diskImage = diskImage;
            this.componentIdSelector = componentIdSelector;
            this.componentIdFilter = componentIdFilter;
            this.hardwareTypeFilter = hardwareTypeFilter;
            this.availableFilter = availableFilter;
            this.sliverType = sliverType;
            this.exclusive = exclusive;
        }

        @JsonCreator
        public static NodeInfo createNodeInfoFromJsonObject(@Nonnull Map<String,Object> props) {
            //TODO: Check if types are correct etc
            int count = (int) props.get("count");
            List<String> componentIdSelector = null;
            Object componentIdSelectorObj = props.get("componentIdSelector");
            if (componentIdSelectorObj != null) {
                if (componentIdSelectorObj instanceof Collection) {
                    componentIdSelector = new ArrayList((Collection) componentIdSelectorObj);
                }
                if (componentIdSelectorObj instanceof String) {
                    componentIdSelector = Collections.singletonList((String) componentIdSelectorObj);
                }
                assert componentIdSelector != null;
            }
            List<String> componentIdFilter = null;
            Object componentIdFilterObj = props.get("componentIdFilter");
            if (componentIdFilterObj != null) {
                if (componentIdFilterObj instanceof Collection) {
                    componentIdFilter = new ArrayList((Collection) componentIdFilterObj);
                }
                if (componentIdFilterObj instanceof String) {
                    componentIdFilter = Collections.singletonList((String) componentIdFilterObj);
                }
                assert componentIdFilter != null;
            }
            List<String> hardwareTypeFilter = null;
            Object hardwareTypeFilterObj = props.get("hardwareTypeFilter");
            if (hardwareTypeFilterObj != null) {
                if (hardwareTypeFilterObj instanceof Collection) {
                    hardwareTypeFilter = new ArrayList((Collection) hardwareTypeFilterObj);
                }
                if (hardwareTypeFilterObj instanceof String) {
                    hardwareTypeFilter = Collections.singletonList((String) hardwareTypeFilterObj);
                }
                assert hardwareTypeFilter != null;
            }
            return new NodeInfo(count,
                    (String) props.get("clientIdBase"),
                    (String) props.get("ansibleGroup"),
                    (String) props.get("diskImage"),
                    componentIdSelector,
                    componentIdFilter,
                    (String) props.get("sliverType"),
                    hardwareTypeFilter,
                    (Boolean) props.get("availableFilter"),
                    (Boolean) props.get("exclusive"));
        }

        public int getCount() {
            return count;
        }

        @Nonnull
        public String getClientIdBase() {
            return clientIdBase;
        }

        @Nullable
        public String getAnsibleGroup() {
            return ansibleGroup;
        }

        @Nullable
        public String getDiskImage() {
            return diskImage;
        }

        @Nullable
        public List<String> getComponentIdSelector() {
            return componentIdSelector;
        }

        @Nullable
        public List<String> getComponentIdFilter() {
            return componentIdFilter;
        }

        @Nullable
        public String getSliverType() {
            return sliverType;
        }

        @Nullable
        public List<String> getHardwareTypeFilter() {
            return hardwareTypeFilter;
        }

        @Nullable
        public Boolean getAvailableFilter() {
            return availableFilter;
        }

        @Nullable
        public Boolean getExclusive() {
            return exclusive;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (!(o instanceof NodeInfo)) return false;

            NodeInfo nodeInfo = (NodeInfo) o;

            if (count != nodeInfo.count) return false;
            if (!clientIdBase.equals(nodeInfo.clientIdBase)) return false;
            if (ansibleGroup != null ? !ansibleGroup.equals(nodeInfo.ansibleGroup) : nodeInfo.ansibleGroup != null)
                return false;
            if (diskImage != null ? !diskImage.equals(nodeInfo.diskImage) : nodeInfo.diskImage != null) return false;
            if (componentIdSelector != null ? !componentIdSelector.equals(nodeInfo.componentIdSelector) : nodeInfo.componentIdSelector != null)
                return false;
            if (componentIdFilter != null ? !componentIdFilter.equals(nodeInfo.componentIdFilter) : nodeInfo.componentIdFilter != null)
                return false;
            if (sliverType != null ? !sliverType.equals(nodeInfo.sliverType) : nodeInfo.sliverType != null)
                return false;
            if (hardwareTypeFilter != null ? !hardwareTypeFilter.equals(nodeInfo.hardwareTypeFilter) : nodeInfo.hardwareTypeFilter != null)
                return false;
            if (availableFilter != null ? !availableFilter.equals(nodeInfo.availableFilter) : nodeInfo.availableFilter != null)
                return false;
            return exclusive != null ? exclusive.equals(nodeInfo.exclusive) : nodeInfo.exclusive == null;
        }

        @Override
        public int hashCode() {
            int result = count;
            result = 31 * result + clientIdBase.hashCode();
            result = 31 * result + (ansibleGroup != null ? ansibleGroup.hashCode() : 0);
            result = 31 * result + (diskImage != null ? diskImage.hashCode() : 0);
            result = 31 * result + (componentIdSelector != null ? componentIdSelector.hashCode() : 0);
            result = 31 * result + (componentIdFilter != null ? componentIdFilter.hashCode() : 0);
            result = 31 * result + (sliverType != null ? sliverType.hashCode() : 0);
            result = 31 * result + (hardwareTypeFilter != null ? hardwareTypeFilter.hashCode() : 0);
            result = 31 * result + (availableFilter != null ? availableFilter.hashCode() : 0);
            result = 31 * result + (exclusive != null ? exclusive.hashCode() : 0);
            return result;
        }

        @Override
        public String toString() {
            return "NodeInfo{" +
                    "count=" + count +
                    ", clientIdBase='" + clientIdBase + '\'' +
                    ", ansibleGroup='" + ansibleGroup + '\'' +
                    ", diskImage='" + diskImage + '\'' +
                    ", componentIdSelector=" + componentIdSelector +
                    ", componentIdFilter=" + componentIdFilter +
                    ", sliverType='" + sliverType + '\'' +
                    ", hardwareTypeFilter=" + hardwareTypeFilter +
                    ", availableFilter=" + availableFilter +
                    ", exclusive=" + exclusive +
                    '}';
        }
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof AdvertisementBasedRspecGeneratorConfig)) return false;

        AdvertisementBasedRspecGeneratorConfig that = (AdvertisementBasedRspecGeneratorConfig) o;

        return nodes.equals(that.nodes);
    }

    @Override
    public int hashCode() {
        return nodes.hashCode();
    }

    @Override
    public String toString() {
        return "AdvertisementBasedRspecGeneratorConfig{" +
                "nodes=" + nodes +
                '}';
    }
}
